/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui;
import javafx.ui.Widget;
import javafx.ui.Border;
import javafx.ui.HorizontalScrollBarPolicy;
import javafx.ui.VerticalScrollBarPolicy;
import javax.swing.event.ChangeListener;
import java.awt.Cursor;
import java.awt.Color as AWTColor;

/**
 * Interface for widgets that support scrollbars and scrolling. Encapsulates
 * javax.swing.JScrollPane and javax.swing.JViewport.
 */

public class ScrollableWidget extends Widget {
    private attribute viewChangeListener: ChangeListener;
    protected attribute scrollpane: <<javax.swing.JScrollPane>>;

    /**
     * Adds a child that will appear as a column on to the left or right
     * of the main view of the scrollpane depending on the current component orientation.
     */
    public attribute rowHeader:Widget?;

    /**
     * Adds a child that will appear as a row at the top or bottom
     * of the main view of the scrollpane depending on the current component orientation.
     */
    public attribute columnHeader:Widget?;

    /*
     * Adds a child that will appear in lower left corner of the scroll pane
     * if there's room. 
     */
    public attribute lowerLeftCorner:Widget?;

    /*
     * Adds a child that will appear in lower right corner of the scroll pane
     * if there's room. 
     */
    public attribute lowerRightCorner:Widget?;

    /*
     * Adds a child that will appear in upper left corner of the scroll pane
     * if there's room. 
     */
    public attribute upperLeftCorner:Widget?;

    /*
     * Adds a child that will appear in upper right corner of the scroll pane
     * if there's room. 
     */
    public attribute upperRightCorner:Widget?;

    /**
     * Sets the border of the scroll pane's viewport
     */
    public attribute viewportBorder:Border?;

    /**
     * Sets the scroll pane's border
     */
    public attribute scrollPaneBorder:Border?;

    /**
     * Optional handler for changes to the scroll pane's view's location.
     */
    public attribute onViewChange: function()?;
    
    /** The display policy for the horizontal scrollbar. Defaults to AS_NEEDED.*/
    public attribute horizontalScrollBarPolicy: HorizontalScrollBarPolicy;

    public attribute scrollBarBackground: AbstractColor?;
    protected attribute awtScrollBarBackground: AWTColor?;
    /** The display policy for the vertical scrollbar. Defaults to AS_NEEDED.*/
    public attribute verticalScrollBarPolicy: VerticalScrollBarPolicy;
    protected operation createView(): <<javax.swing.JComponent>>;
    protected operation createScrollPane(view:<<javax.swing.JComponent>>): <<javax.swing.JScrollPane>>;
    private operation installViewChangeListener();
    
    public operation getViewRect(): <<java.awt.Rectangle>>;
    // hide it for now - buggy? <http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6333318>
    private operation scrollRectToVisible(<<java.awt.Rectangle>>);
    public operation getViewPosition(): <<java.awt.Point>>;
    public operation setViewPosition(<<java.awt.Point>>);
}

attribute ScrollableWidget.awtScrollBarBackground = bind scrollBarBackground.getColor();
trigger on ScrollableWidget.awtScrollBarBackground = value {
    scrollpane.getVerticalScrollBar().setBackground(value);
    scrollpane.getHorizontalScrollBar().setBackground(value);
}

operation ScrollableWidget.onSetOpaque(value:Boolean) {
    scrollpane.setOpaque(value);
    scrollpane.getVerticalScrollBar().setOpaque(value);
    scrollpane.getHorizontalScrollBar().setOpaque(value);
    scrollpane.getViewport().setOpaque(value);
    var comp = scrollpane.getViewport().getView();
    if (comp instanceof <<javax.swing.JComponent>>) {
        ((<<javax.swing.JComponent>>)comp).setOpaque(value);    
    }
}

operation ScrollableWidget.createScrollPane(view:<<javax.swing.JComponent>>) {
    var sp = new <<net.java.javafx.ui.F3ScrollPane>>();
    //sp.setViewport(vp);
    //sp.setViewport(new <<javax.swing.JViewport>>);
    sp.setViewportView(view);
    return sp;
}

operation ScrollableWidget.createComponent() {
    scrollpane = this.createScrollPane(this.createView());
    scrollpane.setOpaque(false);
    scrollpane.getViewport().setOpaque(false);
    scrollpane.setHorizontalScrollBarPolicy(horizontalScrollBarPolicy.id);
    scrollpane.setVerticalScrollBarPolicy(verticalScrollBarPolicy.id);
    scrollpane.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    if (rowHeader <> null) {
        scrollpane.setRowHeaderView(rowHeader.getNonScrollPaneComponent());
    }
    if (columnHeader <> null) {
        scrollpane.setColumnHeaderView(columnHeader.getNonScrollPaneComponent());
    }
    if (lowerLeftCorner <> null) {
        scrollpane.setCorner(scrollpane.LOWER_LEFT_CORNER,
                             lowerLeftCorner.getNonScrollPaneComponent());
    }
    if (lowerRightCorner <> null) {
        scrollpane.setCorner(scrollpane.LOWER_RIGHT_CORNER,
                             lowerRightCorner.getNonScrollPaneComponent());
    }
    if (upperLeftCorner <> null) {
        scrollpane.setCorner(scrollpane.UPPER_LEFT_CORNER,
                             upperLeftCorner.getNonScrollPaneComponent());
    }
    if (upperRightCorner <> null) {
        scrollpane.setCorner(scrollpane.UPPER_RIGHT_CORNER,
                             upperRightCorner.getNonScrollPaneComponent());
    }
    if (viewportBorder <> null) {
        scrollpane.setViewportBorder(viewportBorder.getBorder());
    }
    if (scrollPaneBorder <> null) {
        scrollpane.setBorder(scrollPaneBorder.getBorder());
    }
    if (onViewChange <> null) {
        this.installViewChangeListener();
    }
    if (awtScrollBarBackground <> null) {
        scrollpane.getVerticalScrollBar().setBackground(awtScrollBarBackground);
       scrollpane.getHorizontalScrollBar().setBackground(awtScrollBarBackground);
    }
    return scrollpane;
}

operation ScrollableWidget.installViewChangeListener() {
    if (viewChangeListener == null) {
        var self = this;
        viewChangeListener = new ChangeListener() {
                operation stateChanged(e) {
                    (self.onViewChange)();
                }
            };
        scrollpane.getViewport().addChangeListener(viewChangeListener);
    }
}

operation ScrollableWidget.getViewRect() {
    if (scrollpane <> null) {
        return new <<java.awt.Rectangle>>(scrollpane.getViewport().getViewRect());
    } else {
        return new <<java.awt.Rectangle>>();
    }
}

// <http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6333318>
operation ScrollableWidget.scrollRectToVisible(rect) {
    if (scrollpane <> null) {
        if (rect <> null) {
            scrollpane.getViewport().scrollRectToVisible(rect);
        }
    }
}

operation ScrollableWidget.getViewPosition() {
    if (scrollpane <> null) {
        return new <<java.awt.Point>>(scrollpane.getViewport().getViewPosition());
    } else {
        return new <<java.awt.Point>>();
    }
}

operation ScrollableWidget.setViewPosition(point) {
    if (scrollpane <> null) {
        if (point <> null) {
println(<<java.lang.Thread>>.currentThread());
            println("setting viewPosition to {point}");
            scrollpane.getViewport().setViewPosition(point);
        }
    }
}

trigger on ScrollableWidget.onViewChange = value {
    if (scrollpane <> null) {
        if (value <> null) {
            this.installViewChangeListener();
        } else {
            scrollpane.getViewport().removeChangeListener(viewChangeListener);
        }
    }
}

attribute ScrollableWidget.horizontalScrollBarPolicy =
        AS_NEEDED:HorizontalScrollBarPolicy;

attribute ScrollableWidget.verticalScrollBarPolicy = 
        AS_NEEDED:VerticalScrollBarPolicy;


trigger on ScrollableWidget.verticalScrollBarPolicy[old] = newValue {
    if (scrollpane <> null) {
        var n = verticalScrollBarPolicy.id;
        scrollpane.setVerticalScrollBarPolicy(n);
    }
}

trigger on ScrollableWidget.horizontalScrollBarPolicy[old] = newValue {
    if (scrollpane <> null) {
        var n = horizontalScrollBarPolicy.id;
        scrollpane.setHorizontalScrollBarPolicy(n);
    }
}


trigger on (ScrollableWidget.rowHeader = w) {
    if (scrollpane <> null) {
        var x = (<<java.awt.Component>>)w.getNonScrollPaneComponent();
        scrollpane.setRowHeaderView(x);
    }
}

trigger on (ScrollableWidget.columnHeader = w) {
    if (scrollpane <> null) {
        var x = (<<java.awt.Component>>)w.getNonScrollPaneComponent();
        scrollpane.setColumnHeaderView(x);
    }
}

trigger on (ScrollableWidget.upperRightCorner = w) {
    if (scrollpane <> null) {
        var x = (<<java.awt.Component>>)w.getNonScrollPaneComponent();
        scrollpane.setCorner(scrollpane.UPPER_RIGHT_CORNER, x);
    }
}

trigger on (ScrollableWidget.upperLeftCorner = w) {
    if (scrollpane <> null) {
        var x = (<<java.awt.Component>>)w.getNonScrollPaneComponent();
        scrollpane.setCorner(scrollpane.UPPER_LEFT_CORNER, x);
    }
}

trigger on (ScrollableWidget.lowerLeftCorner = w) {
    if (scrollpane <> null) {
        var x = (<<java.awt.Component>>)w.getNonScrollPaneComponent();
        scrollpane.setCorner(scrollpane.LOWER_LEFT_CORNER, x);
    }
}

trigger on (ScrollableWidget.lowerRightCorner = w) {
    if (scrollpane <> null) {
        var x = (<<java.awt.Component>>)w.getNonScrollPaneComponent();
        scrollpane.setCorner(scrollpane.LOWER_RIGHT_CORNER, x);
    }
}

trigger on (ScrollableWidget.viewportBorder = b) {
    if (scrollpane <> null) {
        scrollpane.setViewportBorder(b.getBorder());
    }
}

trigger on (ScrollableWidget.scrollPaneBorder = b) {
    if (scrollpane <> null) {
        scrollpane.setBorder(b.getBorder());
    }
}
