/*
 *  $Id$
 * 
 *  Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 *  SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package javafx.ui.canvas;
import javafx.ui.canvas.VisualNode;
import javafx.ui.canvas.AbstractPathElement;
import net.java.javafx.jazz.component.ZShape;
import net.java.javafx.jazz.event.*;
import javafx.ui.XY;
import java.awt.BasicStroke;
import java.awt.geom.GeneralPath;
import java.awt.Shape as JShape;
import java.awt.geom.Point2D as JPoint;
import net.java.javafx.ui.batik.PathLength;
import java.lang.Math;
import javax.swing.event.ChangeListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
/**
 * The <code>Shape</code> interface provides definitions for objects 
 * that represent some form of geometric shape.
 */

public class Shape extends VisualNode, AbstractPathElement {
    protected attribute zshape: ZShape;
    protected operation createShape(): ZShape;
    public operation getShape(): ZShape;
    public operation getTransformedShape(): JShape;
    public attribute outline: Boolean;
    public operation length(): Number;
    public operation pointAt(length: Number): JPoint;
    public operation angleAt(length: Number): Number;
    public operation transformAt(length: Number): Transform*;
    public operation toPath(): Path;
    public attribute fillRule: FillRule;
    private attribute pathLength: PathLength?;
    private attribute awtTransformedShape: JShape?;
    private attribute awtShape: JShape?;
    private attribute awtTransform: AffineTransform?;
    private attribute transformListener: ZTransformListener;
    private operation getAWTTransformedShape(t:AffineTransform, s:JShape):JShape;
    private operation getAWTShape(): JShape;
    ...
}

attribute Shape.awtTransformedShape = bind lazy getAWTTransformedShape(awtTransform, awtShape);

attribute Shape.pathLength = bind lazy new PathLength(awtTransformedShape);

operation Shape.pointAt(length: Number) {
    var shape = this.getTransformedShape();
    if (shape <> null) {
        return pathLength.pointAtLength(length);
    }
    return null;
}

operation Shape.angleAt(length: Number) {
    var shape = this.getTransformedShape();
    if (shape <> null) {
        var angle = pathLength.angleAtLength(length);
        return Math.toDegrees(angle);
    }
    return null;
}

operation Shape.length() {
    var shape = this.getTransformedShape();
    if (shape <> null) {
        return pathLength.lengthOfPath();
    }
    return null;
}

attribute Shape.outline = false;

function Shape.transformAt(length: Number) {
    var pt = pointAt(length);
    var angle = angleAt(length);
    return CompositeTransform {
        transforms: bind [translate(pt.x, pt.y), rotate(angle, 0, 0)]
    };
}

/*
function Shape.transformAtUnitInterval(ui: Number) {
    return transformAt(ui*length());
}
*/

operation Shape.addTo(gp:GeneralPath) {
    var tshape = this.getTransformedShape();
    if (outline) {
        var s = new BasicStroke(strokeWidth,
                                strokeLineCap.id,
                                strokeLineJoin.id,
                                strokeMiterLimit,
                                strokeDashArray,
                                strokeDashOffset);
        gp.append(s.createStrokedShape(tshape), false);
    } else {
        gp.append(tshape, false);
    }
}

operation Shape.createVisualNode() {
    return this.getShape();
}

operation Shape.getShape() {
    if (zshape == null) {
        zshape = this.createShape();
        var self = this;
    }
    return zshape;
}

operation Shape.getAWTTransformedShape(t:AffineTransform, s:JShape) {
    return t.createTransformedShape(s);
}

operation Shape.getAWTShape() {
    getNode();
    if (transformListener == null) {
        var self = this;
        zshape.addChangeListener(new ChangeListener() {
                operation stateChanged(e) {
                    self.awtShape = null;
                    self.awtShape = self.zshape.getShape();
                }
        });
        transformListener = new ZTransformListener() {
                operation transformChanged(e) {
                    self.awtTransform = self.tg.getTransform();
                }
            };
        tg.addTransformListener(transformListener);
    }
    awtTransform = tg.getTransform();
    awtShape = getShape().getShape();
    return awtTransformedShape;
}

function Shape.getTransformedShape() {
    return if awtTransformedShape == null then getAWTShape() else awtTransformedShape;
}

attribute Shape.selectable = false;

operation Shape.toPath() {
   var p = new PathLength(getTransformedShape());
   var segs = p.getSegments();
   var path = Path {};
   for (seg in segs) {
       if (seg.getSegType() == PathIterator.SEG_MOVETO) {
           insert MoveTo {x: seg.getX(), y: seg.getY()} into path.d;
       } else if (seg.getSegType() == PathIterator.SEG_LINETO) {
           insert LineTo {x: seg.getX(), y: seg.getY()} into path.d;
       } else {
           // close
           insert ClosePath into path.d;
       }
   }
   return path;
}
